@@ -40,17 +40,27 @@ pub(crate) enum ValueErrorKind {
     UnnecessaryTypeParam(Span),
     /// Inner expression of widen is wider than target width
     InvalidWiden(ConcreteValueType, ConcreteValueType),
+    /// Optional Type is not allowed
+    OptionNotAllowed(ConcreteValueType),
 }
 
 /// The [AbstractValueType] is used during inference and represents a value within the type lattice
 #[derive(Debug, Clone, Copy, Eq, PartialEq)]
 pub(crate) enum AbstractValueType {
     Any,
+    /// A numeric value which is either an integer or a float
     Numeric,
+    /// Either a signed or and unsigned integer
     Integer,
-    SInteger(u32),
-    UInteger(u32),
-    Float(u32),
+    /// An signed integer of arbitrary size
+    SInteger,
+    SizedSInteger(u32),
+    /// An unsigned integer of arbitrary size
+    UInteger,
+    SizedUInteger(u32),
+    /// A float of arbitrary size
+    Float,
+    SizedFloat(u32),
     Bool,
     AnyTuple,
     Tuple(usize),
@@ -85,26 +95,36 @@ impl Variant for AbstractValueType {
             (other, Any) => Ok((other, lhs.least_arity)),
             (Numeric, Numeric) => Ok((Numeric, 0)),
             (Integer, Integer) => Ok((Integer, 0)),
-            (SInteger(l), SInteger(r)) if l == r => Ok((SInteger(l), 0)),
-            (SInteger(1), SInteger(r)) | (SInteger(r), SInteger(1)) => Ok((SInteger(r), 0)),
-            (SInteger(_), SInteger(_)) => Err(TypeClash(lhs.variant, rhs.variant)),
-            (UInteger(l), UInteger(r)) if l == r => Ok((UInteger(l), 0)),
-            (UInteger(1), UInteger(r)) | (UInteger(r), UInteger(1)) => Ok((UInteger(r), 0)),
-            (UInteger(_), UInteger(_)) => Err(TypeClash(lhs.variant, rhs.variant)),
-            (Float(l), Float(r)) if l == r => Ok((Float(l), 0)),
-            (Float(1), Float(r)) | (Float(r), Float(1)) => Ok((Float(r), 0)),
-            (Float(_), Float(_)) => Err(TypeClash(lhs.variant, rhs.variant)),
+            (SInteger, SInteger) => Ok((SInteger, 0)),
+            (UInteger, UInteger) => Ok((UInteger, 0)),
+            (Float, Float) => Ok((Float, 0)),
+            (SInteger, SizedSInteger(x)) | (SizedSInteger(x), SInteger) => Ok((SizedSInteger(x), 0)),
+            (SizedSInteger(l), SizedSInteger(r)) if l == r => Ok((SizedSInteger(l), 0)),
+            (SizedSInteger(_), SizedSInteger(_)) => Err(TypeClash(lhs.variant, rhs.variant)),
+            (UInteger, SizedUInteger(x)) | (SizedUInteger(x), UInteger) => Ok((SizedUInteger(x), 0)),
+            (SizedUInteger(l), SizedUInteger(r)) if l == r => Ok((SizedUInteger(l), 0)),
+            (SizedUInteger(_), SizedUInteger(_)) => Err(TypeClash(lhs.variant, rhs.variant)),
+            (Float, SizedFloat(x)) | (SizedFloat(x), Float) => Ok((SizedFloat(x), 0)),
+            (SizedFloat(l), SizedFloat(r)) if l == r => Ok((SizedFloat(l), 0)),
+            (SizedFloat(_), SizedFloat(_)) => Err(TypeClash(lhs.variant, rhs.variant)),
             (Bool, Bool) => Ok((Bool, 0)),
             (Bool, _) | (_, Bool) => Err(TypeClash(lhs.variant, rhs.variant)),
             (Numeric, Integer) | (Integer, Numeric) => Ok((Integer, 0)),
-            (Numeric, SInteger(w)) | (SInteger(w), Numeric) => Ok((SInteger(w), 0)),
-            (Numeric, UInteger(w)) | (UInteger(w), Numeric) => Ok((UInteger(w), 0)),
-            (Numeric, Float(i)) | (Float(i), Numeric) => Ok((Float(i), 0)),
-            (Integer, SInteger(x)) | (SInteger(x), Integer) => Ok((SInteger(x), 0)),
-            (Integer, UInteger(x)) | (UInteger(x), Integer) => Ok((UInteger(x), 0)),
+            (Numeric, SInteger) | (SInteger, Numeric) => Ok((SInteger, 0)),
+            (Numeric, SizedSInteger(w)) | (SizedSInteger(w), Numeric) => Ok((SizedSInteger(w), 0)),
+            (Numeric, UInteger) | (UInteger, Numeric) => Ok((UInteger, 0)),
+            (Numeric, SizedUInteger(w)) | (SizedUInteger(w), Numeric) => Ok((SizedUInteger(w), 0)),
+            (Numeric, Float) | (Float, Numeric) => Ok((Float, 0)),
+            (Numeric, SizedFloat(i)) | (SizedFloat(i), Numeric) => Ok((SizedFloat(i), 0)),
+            (Integer, SInteger) | (SInteger, Integer) => Ok((SInteger, 0)),
+            (Integer, UInteger) | (UInteger, Integer) => Ok((UInteger, 0)),
+            (Integer, SizedSInteger(x)) | (SizedSInteger(x), Integer) => Ok((SizedSInteger(x), 0)),
+            (Integer, SizedUInteger(x)) | (SizedUInteger(x), Integer) => Ok((SizedUInteger(x), 0)),
             (Integer, _) | (_, Integer) => Err(TypeClash(lhs.variant, rhs.variant)),
-            (SInteger(_), _) | (_, SInteger(_)) => Err(TypeClash(lhs.variant, rhs.variant)),
-            (UInteger(_), _) | (_, UInteger(_)) => Err(TypeClash(lhs.variant, rhs.variant)),
+            (SInteger, _) | (_, SInteger) => Err(TypeClash(lhs.variant, rhs.variant)),
+            (SizedSInteger(_), _) | (_, SizedSInteger(_)) => Err(TypeClash(lhs.variant, rhs.variant)),
+            (UInteger, _) | (_, UInteger) => Err(TypeClash(lhs.variant, rhs.variant)),
+            (SizedUInteger(_), _) | (_, SizedUInteger(_)) => Err(TypeClash(lhs.variant, rhs.variant)),
             (AnyTuple, AnyTuple) => Ok((AnyTuple, max(lhs.least_arity, rhs.least_arity))),
             (AnyTuple, Tuple(size)) => tuple_meet(lhs.least_arity, size),
             (Tuple(size), AnyTuple) => tuple_meet(rhs.least_arity, size),
@@ -134,9 +154,8 @@ impl Variant for AbstractValueType {
             Any | AnyTuple => Arity::Variable,
             Tuple(x) => Arity::Fixed(*x),
             Option => Arity::Fixed(1),
-            Numeric | Integer | SInteger(_) | UInteger(_) | Float(_) | Bool | Sequence | String | Bytes => {
-                Arity::Fixed(0)
-            },
+            Numeric | Integer | SInteger | SizedSInteger(_) | UInteger | SizedUInteger(_) | Float | SizedFloat(_)
+            | Bool | Sequence | String | Bytes => Arity::Fixed(0),
         }
     }
 }
@@ -149,19 +168,22 @@ impl Constructable for AbstractValueType {
         match self {
             AbstractValueType::Any => Err(CannotReify(*self)),
             AbstractValueType::AnyTuple => Err(CannotReify(*self)),
-            AbstractValueType::SInteger(w) if *w <= 8 => Ok(ConcreteValueType::Integer8),
-            AbstractValueType::SInteger(w) if *w <= 16 => Ok(ConcreteValueType::Integer16),
-            AbstractValueType::SInteger(w) if *w <= 32 => Ok(ConcreteValueType::Integer32),
-            AbstractValueType::SInteger(w) if *w <= 64 => Ok(ConcreteValueType::Integer64),
-            AbstractValueType::SInteger(_) => Err(ReificationTooWide(*self)),
-            AbstractValueType::UInteger(w) if *w <= 8 => Ok(ConcreteValueType::UInteger8),
-            AbstractValueType::UInteger(w) if *w <= 16 => Ok(ConcreteValueType::UInteger16),
-            AbstractValueType::UInteger(w) if *w <= 32 => Ok(ConcreteValueType::UInteger32),
-            AbstractValueType::UInteger(w) if *w <= 64 => Ok(ConcreteValueType::UInteger64),
-            AbstractValueType::UInteger(_) => Err(ReificationTooWide(*self)),
-            AbstractValueType::Float(w) if *w <= 32 => Ok(ConcreteValueType::Float32),
-            AbstractValueType::Float(w) if *w <= 64 => Ok(ConcreteValueType::Float64),
-            AbstractValueType::Float(_) => Err(ReificationTooWide(*self)),
+            AbstractValueType::SInteger => Ok(ConcreteValueType::Integer64),
+            AbstractValueType::SizedSInteger(w) if *w <= 8 => Ok(ConcreteValueType::Integer8),
+            AbstractValueType::SizedSInteger(w) if *w <= 16 => Ok(ConcreteValueType::Integer16),
+            AbstractValueType::SizedSInteger(w) if *w <= 32 => Ok(ConcreteValueType::Integer32),
+            AbstractValueType::SizedSInteger(w) if *w <= 64 => Ok(ConcreteValueType::Integer64),
+            AbstractValueType::SizedSInteger(_) => Err(ReificationTooWide(*self)),
+            AbstractValueType::UInteger => Ok(ConcreteValueType::UInteger64),
+            AbstractValueType::SizedUInteger(w) if *w <= 8 => Ok(ConcreteValueType::UInteger8),
+            AbstractValueType::SizedUInteger(w) if *w <= 16 => Ok(ConcreteValueType::UInteger16),
+            AbstractValueType::SizedUInteger(w) if *w <= 32 => Ok(ConcreteValueType::UInteger32),
+            AbstractValueType::SizedUInteger(w) if *w <= 64 => Ok(ConcreteValueType::UInteger64),
+            AbstractValueType::SizedUInteger(_) => Err(ReificationTooWide(*self)),
+            AbstractValueType::Float => Ok(ConcreteValueType::Float32),
+            AbstractValueType::SizedFloat(w) if *w <= 32 => Ok(ConcreteValueType::Float32),
+            AbstractValueType::SizedFloat(w) if *w <= 64 => Ok(ConcreteValueType::Float64),
+            AbstractValueType::SizedFloat(_) => Err(ReificationTooWide(*self)),
             AbstractValueType::Numeric => Err(CannotReify(*self)),
             AbstractValueType::Integer => Ok(ConcreteValueType::Integer64),
             AbstractValueType::Bool => Ok(ConcreteValueType::Bool),
@@ -240,9 +262,12 @@ impl Display for AbstractValueType {
             AbstractValueType::Any => write!(f, "Any"),
             AbstractValueType::Numeric => write!(f, "Numeric"),
             AbstractValueType::Integer => write!(f, "Integer"),
-            AbstractValueType::SInteger(w) => write!(f, "Int({})", *w),
-            AbstractValueType::UInteger(w) => write!(f, "UInt({})", *w),
-            AbstractValueType::Float(w) => write!(f, "Float({})", *w),
+            AbstractValueType::SInteger => write!(f, "Int"),
+            AbstractValueType::SizedSInteger(w) => write!(f, "Int({})", *w),
+            AbstractValueType::UInteger => write!(f, "UInt"),
+            AbstractValueType::SizedUInteger(w) => write!(f, "UInt({})", *w),
+            AbstractValueType::Float => write!(f, "Float"),
+            AbstractValueType::SizedFloat(w) => write!(f, "Float({})", *w),
             AbstractValueType::Bool => write!(f, "Bool"),
             AbstractValueType::AnyTuple => write!(f, "AnyTuple"),
             AbstractValueType::Tuple(w) => write!(f, "{}Tuple", *w),
@@ -495,6 +520,15 @@ impl Emittable for ValueErrorKind {
                     .maybe_add_span_with_label(key1.and_then(|k| spans.get(&k).cloned()), Some(&format!("Widen with traget {} is found here", bound)), false)
                     .maybe_add_span_with_label(key2.and_then(|k| spans.get(&k).cloned()), Some(&format!("Inferred type {} here", inner)), true)
                     .emit()
+            },
+            ValueErrorKind::OptionNotAllowed(ty) => {
+                Diagnostic::error(
+                     handler,
+                "In value type analysis:\nAn optional type is not allowed here."
+                )
+                .maybe_add_span_with_label(key1.and_then(|k| spans.get(&k).cloned()), Some(&format!("Optional type: {} found here", ty)), true)
+                    .add_note("Help: Consider using the default operator to resolve the optional.")
+                .emit()
             }
         }
     }
